package Egg::Model;
#
# Masatoshi Mizuno E<lt>lusheE<64>cpan.orgE<gt>
#
# $Id: Model.pm 226 2008-01-27 10:23:16Z lushe $
#
use strict;
use warnings;
use base qw/ Egg::Base /;

our $VERSION= '3.00';

1;

__END__

=head1 NAME

Egg::Model - Base class for model.

=head1 DESCRIPTION

It is a base class for the model component.

This module has succeeded to L<Egg::Base>.

=head1 SEE ALSO

L<Egg::Release>,
L<Egg::Base>,

=head1 AUTHOR

Masatoshi Mizuno E<lt>lusheE<64>cpan.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2008 Bee Flag, Corp. E<lt>L<http://egg.bomcity.com/>E<gt>, All Rights Reserved.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.6 or,
at your option, any later version of Perl 5 you may have available.

=cut

